part of plot_window;

abstract class Performable {
  Performer get performer;
  //  Iterable<Setting> get settings;
}

abstract class Performer {

  static math.Random _rand = new math.Random();
  static String _GetRandomString([int length=10]) {
    int a = "a".codeUnitAt(0);
    StringBuffer sb = new StringBuffer();
    for (int i=0; i<length; i++) {
      sb.writeCharCode(_rand.nextInt(26)+a);
    }
    return sb.toString();
  }
  
  String _id;
  String get id {
    if (_id==null)
      _id=Performer._GetRandomString();
    return _id;
  }

  
  Element get html;
  void perform();
}
  

class SettingsGroup extends Performer {
  List<Performer> performers;

  LIElement container;

  String name;
  void perform() {
    performers.forEach((Performer s)=>s.perform());
  }
  
  SettingsGroup(this.name) {
    performers = new List<Performer>();
    container = new LIElement();
  }

  void clear() {
    this.performers.clear();
  }

  void populate(Performable performable, [bool reset=false]) {
    print("populate called with $performable");
    if(reset)
      this.performers.clear();
    this.performers.add(performable.performer);
  }

  void addPerformer(Performer performer) {
    this.performers.add(performer);
  }

  Element get html {
    print("settingsgroup html called");
    container.children..clear()
    ..add(new HeadingElement.h2()..text=this.name)
    ..add(new UListElement()..children.addAll(performers.map((Performer h)=>h.html)));
    return container;
  }

  ButtonElement b;
  
  void addButton() {
    b = new ButtonElement()..text="Done";
    container.children.add(b);
  }

  Future run() async {
    addButton();
    print("run called");
    /*    container.children.clear();
    container.children.addAll(performers.map((Setting s)=>s.html));
    var b = new ButtonElement()..text="Done";
    container.children.add(b);
    show(); */
    return await b.onClick.first.then((Event e)=>perform());
  }

}


typedef void Action(dynamic);

abstract class Setting extends Performer {
  String text;
  Action action;
  InputElement inputElement;

  Setting(this.text, this.action);
  
  Element get html {
    return new LIElement()
    ..children.addAll([new LabelElement()..text=text,
      inputElement]);
  }
  
  void perform();
}

class StringSetting extends Setting {
  
  String value;
  
  StringSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new InputElement()..value=value;
  }

  void perform() {
    if(inputElement.value!=value) {
      action(inputElement.value);
    }
  }
}

class NumSetting extends Setting {
  num value;
  
  NumSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new NumberInputElement()..valueAsNumber=value;
  }

  void perform() {
    if(inputElement.valueAsNumber!=value) {
      action(inputElement.valueAsNumber);
    }
  }
}

class DataListSetting extends Setting {
  List values;
  DataListElement dle;
  var current;

  DataListSetting(String text, this.current, this.values, Action action):super(text, action) {
    //    id = values.map((var value)=>"$value").join("_");
    dle = new DataListElement()..id=id;
    dle.children.addAll(values.map((var val)=>new OptionElement()..value="$val"));
    this.inputElement = new TextInputElement()..value="${current}";
  }

  Element get html {
    LIElement li = new LIElement();
    li.children..add(new LabelElement()..text=text)
    ..addAll([inputElement, dle]);
    inputElement.setAttribute("list", id);
    return li;
  }

  void perform() {
    action(inputElement.value);
  }
}


class BoolSetting extends Setting {
  bool value;

  BoolSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new CheckboxInputElement()..checked=value;
  }

  void perform() {
    action(inputElement.checked);
  }
}


class ColorSetting extends Setting {

  String value;
  
  ColorSetting(String text, this.value, Action action):super(text, action) {
    this.inputElement = new InputElement()..value=value
      ..type="color";
  }
    
  void perform() {
    if(inputElement.value!=value) {
      action(inputElement.value);
    }
  }
  
}